
#include <pandora/main.h>

#include <ft2build.h>
#include FT_SYSTEM_STREAM_H

/*****************************************************************************
** the ISO/ANSI standard stream object 
*/

typedef struct FT_StdStreamRec_
{
    FT_StreamRec  stream;
    FILE*         file;
    const char*   pathname;
} FT_StdStreamRec, *FT_StdStream;

/*****************************************************************************
** read bytes from a standard stream 
*/

static FT_ULong ft_std_stream_read( FT_StdStream stream, FT_Byte *buffer, FT_ULong size)
{
   struct acRead read;
  
   if (size < 1) return(0);
   
   read.Buffer = buffer;
   read.Length = size;
   if (Action(AC_Read, stream->file, &read) IS ERR_Okay) {
      return(read.Result);
   }
   else return(0);
}

/*****************************************************************************
** seek the standard stream to a new position 
*/

static FT_Error ft_std_stream_seek( FT_StdStream stream, FT_ULong pos)
{
   if (ActionTags(AC_Seek, stream->file, (DOUBLE)pos, POS_START) IS ERR_Okay) return(FT_Err_Ok);
   else return(FT_Err_Stream_Seek);
}
  
/*****************************************************************************
** close a standard stream 
*/  

static void ft_std_stream_done(FT_StdStream  stream)
{
   Action(AC_Free, stream->file, NULL);
   stream->file     = NULL;
   stream->pathname = NULL;
}

/*****************************************************************************
** open a standard stream from a given pathname 
*/

static void ft_std_stream_init( FT_StdStream stream, const char *pathname)
{
   LONG size;

   if (CreateObject(ID_FILE, NF_CHILD, &stream->file, NULL,
      FID_Location|TSTRING, pathname,
      FID_Flags|TLONG,      FL_READ,
      TAGEND) IS ERR_Okay) {

      GetField(stream->file, FID_Size, FT_LONG, &size);
      FT_STREAM__SIZE(stream) = size;

      stream->pathname = pathname;
      stream->pos      = 0;
   }
   else {
      DPrintF("@ft_std_stream_init:","iso.stream.init: could not open '%s'", pathname));
      FT_XTHROW( FT_Err_Stream_Open );
   }
}                 

/*****************************************************************************
**
*/

static void ft_std_stream_class_init( FT_ClassRec*  _clazz )
{
   FT_StreamClassRec*  clazz = FT_STREAM_CLASS(_clazz);
   
   clazz->stream_read = (FT_Stream_ReadFunc) ft_std_stream_read;
   clazz->stream_seek = (FT_Stream_SeekFunc) ft_std_stream_seek;
}

/****************************************************************************/

static const FT_TypeRec ft_std_stream_type;
{
    "StreamClass",
    NULL,
    sizeof( FT_ClassRec ),
    ft_stream_class_init,
    NULL,
    sizeof( FT_StdStreamRec ),
    ft_std_stream_init,
    ft_std_stream_done,
    NULL,
};

/****************************************************************************/

FT_EXPORT_DEF(FT_Stream) ft_std_stream_new(FT_Memory memory, const char *pathname)
{
   FT_Class  clazz;
    
   clazz = ft_class_from_type( memory, &ft_std_stream_type );
   return (FT_Stream) ft_object_new( clazz, pathname );
}                     

/****************************************************************************/

FT_EXPORT_DEF( void ) ft_std_stream_create(FT_Memory memory, const char *pathname, FT_Stream *astream)
{
   FT_Class  clazz;
 
   clazz = ft_class_from_type( memory, &ft_std_stream_type );
   ft_object_create( clazz, pathname, FT_OBJECT_P(astream) );
}                        

